@@ -7,6 +7,7 @@ import aiohttp
 
 from .client import Client
 from .version import get_server_version
+from .dump import dump_msgs
 
 logging.basicConfig(level=logging.INFO)
 logger = logging.getLogger(__package__)
@@ -19,6 +20,13 @@ def get_arguments() -> argparse.Namespace:
     parser.add_argument(
         "--server-version", action="store_true", help="Print the version of the server"
     )
+    parser.add_argument(
+        "--dump-state", action="store_true", help="Dump the driver state"
+    )
+    parser.add_argument(
+        "--event-timeout",
+        help="How long to listen for events when dumping state",
+    )
     parser.add_argument(
         "url",
         type=str,
@@ -36,6 +44,8 @@ async def main() -> None:
     async with aiohttp.ClientSession() as session:
         if args.server_version:
             await print_version(args, session)
+        elif args.dump_state:
+            await handle_dump_state(args, session)
         else:
             await connect(args, session)
 
@@ -50,13 +60,21 @@ async def print_version(
     print("Home ID:", version.home_id)
 
 
+async def handle_dump_state(
+    args: argparse.Namespace, session: aiohttp.ClientSession
+) -> None:
+    """Dump the state of the server."""
+    timeout = None if args.event_timeout is None else float(args.event_timeout)
+    msgs = await dump_msgs(args.url, session, timeout=timeout)
+    for msg in msgs:
+        print(msg)
+
+
 async def connect(args: argparse.Namespace, session: aiohttp.ClientSession) -> None:
     """Connect to the server."""
     client = Client(args.url, session)
     asyncio.create_task(client.connect())
 
-    setup = False
-
     def log_value_updated(event: dict) -> None:
         node = event["node"]
         value = event["value"]
@@ -74,26 +92,36 @@ async def connect(args: argparse.Namespace, session: aiohttp.ClientSession) -> N
             value.value,
         )
 
-    while True:
-        try:
+    is_initialized = asyncio.Event()
+
+    async def driver_initialized() -> None:
+        """Handle driver init."""
+        assert client.driver is not None  # type
+
+        # Set up listeners on new nodes
+        client.driver.controller.on(
+            "node added",
+            lambda event: event["node"].on("value updated", log_value_updated),
+        )
+
+        # Set up listeners on existing nodes
+        for node in client.driver.controller.nodes.values():
+            node.on("value updated", log_value_updated)
+
+        is_initialized.set()
+
+    client.register_on_initialized(driver_initialized)
+
+    try:
+        await is_initialized.wait()
+
+        while True:
             await asyncio.sleep(0.1)
 
-            if not setup and client.driver:
-                setup = True
-
-                # Set up listeners on new nodes
-                client.driver.controller.on(
-                    "node added",
-                    lambda event: event["node"].on("value updated", log_value_updated),
-                )
-
-                # Set up listeners on existing nodes
-                for node in client.driver.controller.nodes.values():
-                    node.on("value updated", log_value_updated)
-        except asyncio.CancelledError:
-            logger.info("Close requested")
-            await client.disconnect()
-            raise
+    except asyncio.CancelledError:
+        logger.info("Close requested")
+        await client.disconnect()
+        raise
 
 
 if __name__ == "__main__":
